from PyQt5.QtWidgets import QDialog, QPushButton, QApplication, QLabel, qApp, QLineEdit


class UserName(QDialog):
    """
    Класс реализующий стартовый диалог с запросом логина и пароля
    пользователя.
    """
    def __init__(self):
        super().__init__()

        self.press_ok = False

        self.setFixedSize(180, 140)
        self.setWindowTitle('Ввод имени')

        self.label = QLabel('Введите имя пользователя:', self)
        self.label.move(10, 10)
        self.label.setFixedSize(150, 10)

        self.client_name = QLineEdit(self)
        self.client_name.setFixedSize(155, 20)
        self.client_name.move(10, 30)

        self.btn_ok = QPushButton('Войти', self)
        self.btn_ok.move(10, 105)
        self.btn_ok.clicked.connect(self.check_click)

        self.btn_cancel = QPushButton('Отмена', self)
        self.btn_cancel.move(90, 105)
        self.btn_cancel.clicked.connect(qApp.exit)

        self.label_passwd = QLabel('Введите пароль:', self)
        self.label_passwd.move(10, 55)
        self.label_passwd.setFixedSize(150, 15)

        self.client_passwd = QLineEdit(self)
        self.client_passwd.setFixedSize(154, 20)
        self.client_passwd.move(10, 75)
        self.client_passwd.setEchoMode(QLineEdit.Password)

        self.show()

    # Проверяем заполнение поля
    def check_click(self):
        """Функция обработки нажатия кнопки ОК."""
        if self.client_name.text() and self.client_passwd.text():
            self.press_ok = True
            qApp.exit()


if __name__ == '__main__':
    app = QApplication([])
    dial = UserName()
    app.exec_()



import socket
import sys
import time
import threading
import json
import logging
import hashlib
import hmac
import binascii

from PyQt5.QtCore import pyqtSignal, QObject
from common.utils import get_message, send_message
from common.errors import ServerError, IncorrectDataRecivedError
from common.variables import ACTION, PRESENCE, TIME, USER, ACCOUNT_NAME, PUBLIC_KEY, RESPONSE, ERROR, DATA, \
    RESPONSE_511, MESSAGE, SENDER, DESTINATION, MESSAGE_TEXT, GET_CONTACTS, LIST_INFO, PUBLIC_KEY_REQUEST, \
    ADD_CONTACT, REMOVE_CONTACT, EXIT, USERS_REQUEST

sys.path.append('../')

# Инициализация логгера клиента
LOGGER = logging.getLogger('client')
# Объект блокировки сокета и работы с базой данных
sock_lock = threading.Lock()


class ClientTransport(threading.Thread, QObject):
    """
    Класс - Транспорт, отвечает за взаимодействие с сервером.
    Реализует транспортную подсистему клиентского модуля.
    """
    # Сигналы новое сообщение и потеря соединения
    new_message = pyqtSignal(str)
    message_205 = pyqtSignal()
    connection_lost = pyqtSignal()

    def __init__(self, port, ip_address, database, username, passwd, keys):
        threading.Thread.__init__(self)
        QObject.__init__(self)

        self.database = database
        # Имя пользователя
        self.username = username
        # Сокет для работы с сервером
        self.transport = None
        # Пароль
        self.password = passwd
        # Набор ключей для шифрования
        self.keys = keys
        # Устанавливаем соединение:
        self.connection_init(port, ip_address)
        # Обновление таблиц известных пользователей и контактов
        try:
            self.user_list_request()
            self.contacts_list_request()
        except OSError as e:
            if e.errno:
                LOGGER.critical(f'Соединение с сервером потеряно')
                raise ServerError(f'Соедниение с сервером потеряно')
            LOGGER.error('Таймаут соединения при обновлении списков пользователей')
        except json.JSONDecodeError:
            LOGGER.critical(f'Соединение с сервером потеряно')
            raise ServerError(f'Соединение с сервером потеряно')
        self.running = True

    def connection_init(self, port, ip):
        """Функция инициализации соединения с сервером."""
        # Инициализация сокета и сообщение серверу о нашем появлении
        self.transport = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.transport.settimeout(5)

        # Попытка соединения
        connected = False
        for i in range(5):
            LOGGER.info(f'Попытка соединения с сервером №{i+1}')
            try:
                self.transport.connect((ip, port))
            except (OSError, ConnectionRefusedError):
                pass
            else:
                connected = True
                break
            time.sleep(1)

        # Если соединиться с сервером не удалось
        if not connected:
            LOGGER.critical('Не удалось установить соединение с сервером')
            raise ServerError('Не удалось установить соединение с сервером')

        LOGGER.debug('Запускаем авторизацию пользователя')
        # Запускаем процедуру авторизации
        # Получаем хэш пароля
        passwd_bytes = self.password.encode('utf-8')
        salt = self.username.lower().encode('utf-8')
        passwd_hash = hashlib.pbkdf2_hmac('sha512', passwd_bytes, salt, 10000)
        passwd_hash_string = binascii.hexlify(passwd_hash)

        # Получаем публичный ключ и декодируем его из байтов
        pubkey = self.keys.publickey().export_key().decode('ascii')

        # Авторизируемся на сервере
        with sock_lock:
            presense = {
                ACTION: PRESENCE,
                TIME: time.time(),
                USER: {
                    ACCOUNT_NAME: self.username,
                    PUBLIC_KEY: pubkey
                }
            }
            LOGGER.debug(f'Приветственное сообщение: {presense}')
            # Посылаем серверу приветственное сообщение и получаем ответ что всё нормально или получаем исключение.
            try:
                send_message(self.transport, presense)
                ans = get_message(self.transport)
                LOGGER.debug(f'Ответ сервера: {ans}')
                if RESPONSE in ans:
                    if ans[RESPONSE] == 400:
                        raise ServerError(ans[ERROR])
                    elif ans[RESPONSE] == 511:
                        # Если всё нормально, то продолжаем процедуру авторизации.
                        ans_data = ans[DATA]
                        hash = hmac.new(passwd_hash_string, ans_data.encode('utf-8'))
                        digest = hash.digest()
                        my_ans = RESPONSE_511
                        my_ans[DATA] = binascii.b2a_base64(
                            digest).decode('ascii')
                        send_message(self.transport, my_ans)
                        self.process_server_ans(get_message(self.transport))
            except (OSError, json.JSONDecodeError) as e:
                LOGGER.critical(f'Потеряно соединение с сервером.', exc_info=e)
                raise ServerError('Сбой соединения с сервером в процессе авторизации.')
            # Сообщение об удачной установке соединения.
            LOGGER.debug('Соединение с сервером установлено успешно')

    def create_presence(self):
        """Функция генерирует запрос о присутствии клиента."""
        out = {
            ACTION: PRESENCE,
            TIME: time.time(),
            USER: {
                ACCOUNT_NAME: self.username
            }
        }
        LOGGER.debug(f'Сформировано {PRESENCE} сообщение для пользователя {self.username}')
        return out

    def process_ans(self, message):
        """Функция разбирает ответ сервера. Генерирует исключение при ошибке"""
        LOGGER.debug(f'Разбор сообщения от сервера: {message}')
        if RESPONSE in message:
            if message[RESPONSE] == 200:
                return
            elif message[RESPONSE] == 400:
                raise ServerError(f'400 : {message[ERROR]}')
            elif message[RESPONSE] == 205:
                self.user_list_request()
                self.contacts_list_request()
                self.message_205.emit()
            else:
                LOGGER.debug('Неизвестный код подтверждения')

        # Если сообщение от пользователя, добавляем в базу
        elif ACTION in message and message[ACTION] == MESSAGE and SENDER in message and DESTINATION in message \
                and MESSAGE_TEXT in message and message[DESTINATION] == self.username:
            print(f'Получено сообщение от пользователя {message[SENDER]}: {message[MESSAGE_TEXT]}')
            LOGGER.debug(f'Получено сообщение от пользователя {message[SENDER]}: {message[MESSAGE_TEXT]}')

            self.new_message.emit(message)

    def contacts_list_request(self):
        """Функция генерирует запрос и обновление контакт листа с сервера"""
        self.database.contacts_clear()
        LOGGER.debug(f'Запрос контакт листа для пользователся {self.name}')
        req = {
            ACTION: GET_CONTACTS,
            TIME: time.time(),
            USER: self.username
        }
        LOGGER.debug(f'Сформирован запрос {req}')
        with sock_lock:
            send_message(self.transport, req)
            ans = get_message(self.transport)
        LOGGER.debug(f'Получен ответ {ans}')
        if RESPONSE in ans and ans[RESPONSE] == 202:
            for contact in ans[LIST_INFO]:
                self.database.add_contact(contact)
        else:
            LOGGER.error('Не удалось обновить список контактов')

    def user_list_request(self):
        """Функция запроса списка известных пользователей"""
        LOGGER.debug(f'Запрос списка известных пользователей {self.username}')
        req = {
            ACTION: USERS_REQUEST,
            TIME: time.time(),
            ACCOUNT_NAME: self.username
        }
        with sock_lock:
            send_message(self.transport, req)
            ans = get_message(self.transport)
        if RESPONSE in ans and ans[RESPONSE] == 202:
            self.database.add_users(ans[LIST_INFO])
        else:
            LOGGER.error('Не удалось обновить список известных пользователей')

    def key_request(self, user):
        """Функция запрашивающая с сервера публичный ключ пользователя."""
        LOGGER.debug(f'Запрос публичного ключа для {user}')
        req = {
            ACTION: PUBLIC_KEY_REQUEST,
            TIME: time.time(),
            ACCOUNT_NAME: user
        }
        with sock_lock:
            send_message(self.transport, req)
            ans = get_message(self.transport)
        if RESPONSE in ans and ans[RESPONSE] == 511:
            return ans[DATA]
        else:
            LOGGER.error(f'Не удалось получить ключ собеседника {user}.')

    def add_contact(self, contact):
        """Функция добавления пользователя в контакт лист"""
        LOGGER.debug(f'Создание контакта {contact}')
        req = {
            ACTION: ADD_CONTACT,
            TIME: time.time(),
            USER: self.username,
            ACCOUNT_NAME: contact
        }
        with sock_lock:
            send_message(self.transport, req)
            self.process_ans(get_message(self.transport))

    def remove_contact(self, contact):
        """Функция удаления пользователя из контакт листа"""
        LOGGER.debug(f'Удаление контакта {contact}')
        req = {
            ACTION: REMOVE_CONTACT,
            TIME: time.time(),
            USER: self.username,
            ACCOUNT_NAME: contact
        }
        with sock_lock:
            send_message(self.transport, req)
            self.process_ans(get_message(self.transport))

    def transport_shutdown(self):
        """Функция закрытия соединения с отправкой сообщения о выходе"""
        self.running = False
        message = {
            ACTION: EXIT,
            TIME: time.time(),
            ACCOUNT_NAME: self.username
        }
        with sock_lock:
            try:
                send_message(self.transport, message)
            except OSError:
                pass
        LOGGER.debug('Завершение работы транспорта')
        time.sleep(0.5)

    def send_message(self, to_user, message):
        """ Функция отправки сообщения на сервер """
        message_dic = {
            ACTION: MESSAGE,
            SENDER: self.username,
            DESTINATION: to_user,
            TIME: time.time(),
            MESSAGE_TEXT: message
        }
        LOGGER.debug(f'Сформировано сообщение: {message_dic}')
        # Ждем сокета для отправки сообщения
        with sock_lock:
            send_message(self.transport, message_dic)
            self.process_ans(get_message(self.transport))
            LOGGER.info(f'Сообщение отправлено клиенту {to_user}')

    def run(self):
        """Функция содержащая основной цикл работы транспортного потока."""
        LOGGER.debug('Запущен процесс приема сообщений с сервера')
        while self.running:
            time.sleep(1)
            with sock_lock:
                try:
                    self.transport.settimeout(0.5)
                    message = get_message(self.transport)
                except OSError as e:
                    if e.errno:
                        LOGGER.critical(f'Соединение с сервером потеряно')
                        self.running = False
                        self.connection_lost.emit()
                except IncorrectDataRecivedError:
                    LOGGER.error(f'Не удалось декодировать сообщение')
                except (ConnectionError, ConnectionAbortedError, ConnectionResetError, json.JSONDecodeError, TypeError):
                    LOGGER.critical(f'Соединение с сервером потеряно')
                    self.running = False
                    self.connection_lost.emit()
                finally:
                    self.transport.settimeout(1)

        # Если сообщение получено, то вызываем функцию обработчик:
        if message:
            LOGGER.debug(f'Принято сообщение с сервера: {message}')
            self.process_ans(message)



import base64
import sys
import json
import logging
from PyQt5.QtWidgets import QMainWindow, qApp, QMessageBox, QApplication, QListView
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QBrush, QColor
from PyQt5.QtCore import pyqtSlot, QEvent, Qt
from Crypto.Cipher import PKCS1_OAEP
from Crypto.PublicKey import RSA

from client.main_window_ui import Ui_MainClientWindow
from client.add_contact import AddContactDialog
from client.del_contact import DelContactDialog

from common.errors import ServerError
from common.variables import MESSAGE_TEXT, SENDER

sys.path.append('../')
LOGGER = logging.getLogger('client')


class ClientMainWindow(QMainWindow):
    """
    Класс - основное окно пользователя.
    Содержит всю основную логику работы клиентского модуля.
    Конфигурация окна создана в QTDesigner и загружается из
    конвертированого файла main_window_ui.py
    """
    def __init__(self, database, transport, keys):
        super().__init__()
        # Основные переменные
        self.database = database
        self.transport = transport
        # объект - дешифорвщик сообщений с предзагруженным ключём
        self.decrypter = PKCS1_OAEP.new(keys)
        # Загрузка конфигурации из дизайнера
        self.ui = Ui_MainClientWindow()
        self.ui.setupUi(self)
        # Кнопка выход
        self.ui.menu_exit.triggered.connect(qApp.exit)
        # Кнопка отправки сообщения
        self.ui.btn_send.clicked.connect(self.send_message)
        # Кнопки добавления контакта
        self.ui.btn_add_contact.clicked.connect(self.add_contact_window)
        self.ui.menu_add_contact.triggered.connect(self.add_contact_window)
        # Кнопки удаления контакта
        self.ui.btn_remove_contact.clicked.connect(self.delete_contact_window)
        self.ui.menu_del_contact.triggered.connect(self.delete_contact_window)
        # Дополнительные атрибуты
        self.contacts_model = None
        self.history_model = None
        self.messages = QMessageBox()
        self.current_chat = None
        self.current_chat_key = None
        self.encryptor = None
        self.ui.list_messages.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.ui.list_messages.setWordWrap(True)

        # Даблклик по листу контактов отправляется в обработчик
        self.ui.list_contacts.doubleClicked.connect(self.select_active_user)

        self.contact_list_update()
        self.disable_input()
        self.show()

    def disable_input(self):
        """Функция делающая неактивным поле ввода и кнопки отправки"""
        self.ui.label_new_message.setText('Выберите получателя, дважды кликнув на нем в окне контактов')
        self.ui.text_message.clear()
        if self.history_model:
            self.history_model.clear()

        # Поле ввода и кнопка отправки неактивны до выбора получателя.
        self.ui.btn_send.setDisabled(True)
        self.ui.btn_clear.setDisabled(True)
        self.ui.text_message.setDisabled(True)

        self.encryptor = None
        self.current_chat = None
        self.current_chat_key = None

    def history_list_update(self):
        """
        Функция заполняющая текущий диалог
        историей переписки с текущим собеседником.
        """
        # История сообщений, отсортированная по дате
        list = sorted(self.database.get_history(self.current_chat), key=lambda item: item[3])
        # Если модель не создана
        if not self.history_model:
            self.history_model = QStandardItemModel()
            self.ui.list_messages.setModel(self.history_model)

        # Очищаем от старых записей
        self.history_model.clear()
        length = len(list)
        start_index = 0
        if length > 20:
            start_index = length - 20

        # Заполнение модели записями
        for i in range(start_index, length):
            item = list[i]
            if item[1] == 'in':
                mess = QStandardItem(f'Входящее от {item[3].replace(microsecond=0)}:\n {item[2]}')
                mess.setEditable(False)
                mess.setBackground(QBrush(QColor(255, 213, 213)))
                mess.setTextAlignment(Qt.AlignLeft)
                self.history_model.appendRow(mess)
            else:
                mess = QStandardItem(f'Входящее от {item[3].replace(microsecond=0)}:\n {item[2]}')
                mess.setEditable(False)
                mess.setTextAlignment(Qt.AlignRight)
                mess.setBackground(QBrush(QColor(204, 255, 204)))
                self.history_model.appendRow(mess)
        self.ui.list_messages.scrollToBottom()

    def select_active_user(self):
        """Функция обработки двойного клика"""
        self.current_chat = self.ui.list_contacts.currentIndex().data()
        # Вызов основной функции
        self.set_active_user()

    def set_active_user(self):
        """Функция активации чата с собеседником"""
        # Запрашиваем публичный ключ пользователя и создаём объект шифрования
        try:
            self.current_chat_key = self.transport.key_request(self.current_chat)
            LOGGER.debug(f'Загружен ключ для {self.current_chat}')
            if self.current_chat_key:
                self.encryptor = PKCS1_OAEP.new(RSA.import_key(self.current_chat_key))
        except (OSError, json.JSONDecodeError):
            self.current_chat_key = None
            self.encryptor = None
            LOGGER.debug(f'Не удалось загрузить ключ для {self.current_chat}')

        # Если ключа нет то ошибка, что не удалось начать чат с пользователем
        if not self.current_chat_key:
            self.messages.warning(self, 'Ошибка', 'Для выбранного пользователя нет ключа шифрования.')
            return

        self.ui.label_new_message.setText(f'Введите сообщение для {self.current_chat}: ')
        self.ui.btn_send.setDisabled(False)
        self.ui.btn_clear.setDisabled(False)
        self.ui.text_message.setDisabled(False)
        # Выводим историю по данному пользователю
        self.history_list_update()

    def contact_list_update(self):
        """Функция обновления листа контактов"""
        contact_list = self.database.get_contacts()
        self.contacts_model = QStandardItemModel()
        for i in sorted(contact_list):
            item = QStandardItem(i)
            item.setEditable(False)
            self.contacts_model.appendRow(item)
        self.ui.list_contacts.setModel(self.contacts_model)

    def add_contact_window(self):
        """
        Функция добавления контактов.
        Запускает диалоговое окно для добавления контакта
        """
        global select_dialog
        select_dialog = AddContactDialog(self.transport, self.database)
        select_dialog.btn_ok.clicked.connect(lambda: self.add_contact_action(select_dialog))
        select_dialog.show()

    def add_contact_action(self, item):
        """
        Функция обработчик добавления контакта.
        Обновляет таблицу и список контактов
        """
        new_contact = item.selector.currentText()
        self.add_contact(new_contact)
        item.close()

    def add_contact(self, new_contact):
        """
        Функция добавляющая контакт в серверную и клиентсткую BD.
        После обновления баз данных обновляет и содержимое окна.
        """
        try:
            self.transport.add_contact(new_contact)
        except ServerError as e:
            self.messages.critical(self, 'Ошибка сервера', e.text)
        except OSError as e:
            if e.errno:
                self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером')
                self.close()
            self.messages.critical(self, 'Ошибка', 'Таймаут соединения')
        else:
            self.database.add_contact(new_contact)
            new_contact = QStandardItem(new_contact)
            new_contact.setEditable(False)
            self.contacts_model.appendRow(new_contact)
            LOGGER.info(f'Добавлен контакт: {new_contact}')
            self.messages.information(self, 'ОК', 'Контакт успешно добавлен.')

    def delete_contact_window(self):
        """
        Функция удаления контакта.
        Запускает диалоговое окно для удаления контакта
        """
        global remove_dialog
        remove_dialog = DelContactDialog(self.transport, self.database)
        remove_dialog.btn_ok.clicked.connect(lambda: self.adelete_contact(remove_dialog))
        remove_dialog.show()

    # Обработчик удаления контакта, обновляет таблицу и список контактов
    def delete_contact(self, item):
        """
        Функция удаляет контакт из серверной и клиентсткой BD.
        После обновления баз данных обновляет и содержимое окна.
        """
        select_contact = item.selector.currentText()
        try:
            self.transport.remove_contact(select_contact)
        except ServerError as e:
            self.messages.critical(self, 'Ошибка сервера', e.text)
        except OSError as e:
            if e.errno:
                self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером')
                self.close()
            self.messages.critical(self, 'Ошибка', 'Таймаут соединения')
        else:
            self.database.del_contact(select_contact)
            self.contact_list_update()
            LOGGER.info(f'Удален контакт: {select_contact}')
            self.messages.information(self, 'ОК', 'Контакт успешно добавлен.')
            item.close()
            if select_contact == self.current_chat:
                self.current_chat = None
                self.disable_input()

    def send_message(self):
        """
        Функция отправки сообщения текущему собеседнику.
        Реализует шифрование сообщения и его отправку.
        """
        # Проверка поля на пустоту. Если поле не пустое, то забираем сообщение и очищаем поле
        message_text = self.ui.text_message.toPlainText()
        self.ui.text_message.clear()
        if not message_text:
            return
        message_text_encrypted = self.encryptor.encrypt(message_text.encode('utf8'))
        message_text_encrypted_base64 = base64.b64encode(message_text_encrypted)
        try:
            self.transport.send_message(self.current_chat, message_text_encrypted_base64.decode('ascii'))
            pass
        except ServerError as e:
            self.messages.critical(self, 'Ошибка сервера', e.text)
        except OSError as e:
            if e.errno:
                self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером')
                self.close()
            self.messages.critical(self, 'Ошибка', 'Таймаут соединения')
        except (ConnectionResetError, ConnectionAbortedError):
            self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером')
            self.close()
        else:
            self.database.save_message(self.current_chat, 'out', message_text)
            LOGGER.debug(f'Отравлено сообщение для {self.current_chat}: {message_text}')
            self.history_list_update()

    @pyqtSlot(str)
    def message(self, message):
        """
        Слот обработчик поступаемых сообщений, выполняет дешифровку
        поступаемых сообщений и их сохранение в истории сообщений.
        Запрашивает пользователя если пришло сообщение не от текущего
        собеседника. При необходимости меняет собеседника.
        """
        encrypted_message = base64.b64encode(message[MESSAGE_TEXT])
        try:
            decrypted_message = self.decrypter.decrypt(encrypted_message)
        except (ValueError, TypeError):
            self.messages.warning(self, 'Ошибка', 'Не удалось декодировать сообщение')
            return
        self.database.save_message(self.current_chat, 'in', decrypted_message.decode('utf8'))

        sender = message[SENDER]
        if sender == self.current_chat:
            self.history_list_update()
        else:
            # Проверяем наличие пользователя в контактах
            if self.database.check_contact(sender):
                # Если есть, спрашиваем и желании открыть с ним чат и открываем
                if self.messages.question(self, 'У Вас новое сообщение',\
                                          f'Получено сообщение от {sender}, начать чат?', QMessageBox.Yes,
                                          QMessageBox.No) == QMessageBox.Yes:
                    self.current_chat = sender
                    self.set_active_user()
            else:
                if self.messages.question(self, 'У Вас новое сообщение',\
                                          f'Получено сообщение от {sender}. \n '
                                          f'Пользователя нет в списке контактов. \n '
                                          f'Добавить пользователя в контакты и начать чат?',
                                          QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes:
                    self.add_contact(sender)
                    self.current_chat = sender
                    self.database.save_message(self.current_chat, 'in', decrypted_message.decode('utf8'))
                    self.set_active_user()

    # В случае потери соединения
    @pyqtSlot()
    def connection_lost(self):
        """
        Слот обработчик потери соеднинения с сервером.
        Выдаёт окно предупреждение и завершает работу приложения.
        """
        self.messages.warning(self, 'Сбой соединения', 'Потеряно соединение с сервером')
        self.close()

    @pyqtSlot()
    def signal_205(self):
        """
        Слот выполняющий обновление баз данных по команде сервера.
        """
        if self.current_chat and not self.database.check_user(self.current_chat):
            self.messages.warning(self, 'Упс...', 'Пользователь был удален с сервера')
            self.disabled_input()
            self.current_chat = None
        self.contact_list_update()

    def make_connection(self, trans_obj):
        """Функция обеспечивающая соединение сигналов и слотов."""
        trans_obj.new_message.connect(self.message)
        trans_obj.connection_lost.connect(self.connection_lost)



import sys
import os
import argparse
import logging
from Crypto.PublicKey import RSA
from PyQt5.QtWidgets import QApplication, QMessageBox

import log.client_log_config
from common.variables import DEFAULT_IP_ADDRESS, DEFAULT_PORT
from common.decorators import log
from common.errors import ServerError
from client.database import ClientDatabase
from client.main_window import ClientMainWindow
from client.transport import ClientTransport
from client.start_dialog import UserName

# Инициализация логгера клиента
LOGGER = logging.getLogger('client')


@log
def create_arg_parser():
    """
    Парсер аргументов командной строки, возвращает кортеж из 4 элементов
    адрес сервера, порт, имя пользователя, пароль.
    Выполняет проверку на корректность номера порта.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('addr', default=DEFAULT_IP_ADDRESS, nargs='?')
    parser.add_argument('port', default=DEFAULT_PORT, type=int, nargs='?')
    parser.add_argument('-n', '--name', default=None, nargs='?')
    parser.add_argument('-p', '--password', default='', nargs='?')
    namespace = parser.parse_args(sys.argv[1:])
    server_address = namespace.addr
    server_port = namespace.port
    client_name = namespace.name
    client_passwd = namespace.password

    # Проверяем номер порта
    if not 1023 < server_port < 65536:
        LOGGER.critical(
            f'Запуск сервера с параметром порта {server_port}.'
            f'В качестве порта может быть указано только число в диапазоне от 1024 до 65535.')
        sys.exit(1)

    return server_address, server_port, client_name, client_passwd


if __name__ == '__main__':
    # Проверяем параметры командной строки
    server_address, server_port, client_name, client_passwd = create_arg_parser()
    # Создание клиентского приложения
    client_app = QApplication(sys.argv)
    # Если имя пользователя не было задано, необходимо запросить пользователя.
    start_dialog = UserName()
    if not client_name or not client_passwd:
        client_app.exec_()
        if start_dialog.press_ok:
            client_name = start_dialog.client_name.text()
            client_passwd = start_dialog.client_passwd.text()
            LOGGER.debug(
                f'Залогинился пользователь: {client_name}, с паролем: {client_passwd}')
        else:
            sys.exit(0)

    LOGGER.info(
        f'Запущен клиент с парамертами: адрес сервера: {server_address}, '
        f'порт: {server_port}, имя пользователя: {client_name}')

    # Загрузка ключей из файла, если ключей нет, то гениерируем пару новую
    # dir_path = os.path.dirname(os.path.realpath(__file__))
    dir_path = os.getcwd()
    key_file = os.path.join(dir_path, f'{client_name}.key')
    if not os.path.exists(key_file):
        keys = RSA.generate(2048, os.urandom)
        with open(key_file, 'wb') as key:
            key.write(keys.export_key())
    else:
        with open(key_file, 'rb') as key:
            keys = RSA.import_key(key.read())

    keys.publickey().export_key()

    LOGGER.debug('Ключи для клиентов успешно созданы')

    # Инициализация БД
    database = ClientDatabase(client_name)
    # Инициализация сокета и обмен.
    try:
        transport = ClientTransport(
            server_port,
            server_address,
            database,
            client_name,
            client_passwd,
            keys)
        LOGGER.debug('Транспорт готов')
    except ServerError as error:
        print(error.text)
        message = QMessageBox()
        message.critical(
            start_dialog,
            'Ошибка соединения с сервером',
            error.text)
        LOGGER.error(
            f'Ошибка соединения с сервером. Сервер вернул ошибку: {error.text}')
        sys.exit(1)
    transport.setDaemon(True)
    transport.start()

    # Создаём GUI
    main_window = ClientMainWindow(database, transport, keys)
    main_window.make_connection(transport)
    main_window.setWindowTitle(f'Чат программа - {client_name}')
    client_app.exec_()

    # Раз графическая оболочка закрылась, закрываем транспорт
    transport.transport_shutdown()
    transport.join()
